@@ -62,8 +62,8 @@ int cifs_debug_data_read(char *buf, char **beginBuffer, off_t offset,
 			 int count, int *eof, void *data)
 {
 	struct list_head *tmp;
-	struct smbSesInfo *ses;
-	struct smbTconInfo *tcon;
+	struct cifsSesInfo *ses;
+	struct cifsTconInfo *tcon;
 	int i;
 	int length = 0;
 	char *buf_start = buf;
@@ -81,13 +81,15 @@ int cifs_debug_data_read(char *buf, char **beginBuffer, off_t offset,
 	i = 0;
 	list_for_each(tmp, &GlobalSMBSessionList) {
 		i++;
-		ses = list_entry(tmp, struct smbSesInfo, smbSessionList);
+		ses = list_entry(tmp, struct cifsSesInfo, cifsSessionList);
 		length =
 		    sprintf(buf,
-			    "\n%d) Name: %s  Domain: %s HowManyMounts: %d  ServerOS: %s  ServerNOS: %s  Capabilities: 0x%x ",
+			    "\n%d) Name: %s  Domain: %s HowManyMounts: %d LocalUsersToSameServer: %d\n\t ServerOS: %s  ServerNOS: %s  Capabilities: 0x%x ",
 			    i, ses->serverName, ses->serverDomain,
-			    atomic_read(&ses->inUse), ses->serverOS,
-			    ses->serverNOS, ses->capabilities);
+			    atomic_read(&ses->inUse),
+			    atomic_read(&ses->server->socketUseCount),
+			    ses->serverOS, ses->serverNOS,
+			    ses->capabilities);
 		buf += length;
 	}
 	sprintf(buf, "\n");
@@ -101,7 +103,8 @@ int cifs_debug_data_read(char *buf, char **beginBuffer, off_t offset,
 	list_for_each(tmp, &GlobalTreeConnectionList) {
 		i++;
 		tcon =
-		    list_entry(tmp, struct smbTconInfo, smbConnectionList);
+		    list_entry(tmp, struct cifsTconInfo,
+			       cifsConnectionList);
 		length =
 		    sprintf(buf,
 			    "\n%d) %s UseCount: %d on FS: %s with characteristics: 0x%x Attributes: 0x%x and PathComponentLengthMax: %d",
@@ -173,6 +176,8 @@ static read_proc_t traceSMB_read;
 static write_proc_t traceSMB_write;
 static read_proc_t multiuser_mount_read;
 static write_proc_t multiuser_mount_write;
+static read_proc_t extended_security_read;
+static write_proc_t extended_security_write;
 
 void cifs_proc_init(void)
 {
@@ -214,6 +219,12 @@ void cifs_proc_init(void)
 				   multiuser_mount_read, 0);
 	if (pde)
 		pde->write_proc = multiuser_mount_write;
+
+	pde =
+	    create_proc_read_entry("ExtendedSecurity", 0, proc_fs_cifs,
+				   extended_security_read, 0);
+	if (pde)
+		pde->write_proc = extended_security_write;
 }
 
 void cifs_proc_clean(void)
@@ -380,4 +391,42 @@ static int multiuser_mount_write(struct file *file, const char *buffer,
 	return count;
 }
 
+static int extended_security_read(char *page, char **start, off_t off,
+				  int count, int *eof, void *data)
+{
+	int len;
+
+	len = sprintf(page, "%d\n", extended_security);
+
+	len -= off;
+	*start = page + off;
+
+	if (len > count)
+		len = count;
+	else
+		*eof = 1;
+
+	if (len < 0)
+		len = 0;
+
+	return len;
+}
+static int extended_security_write(struct file *file, const char *buffer,
+				   unsigned long count, void *data)
+{
+	char c;
+	int rc;
+
+	rc = get_user(c, buffer);
+	if (rc)
+		return rc;
+	if (c == '0' || c == 'n' || c == 'N')
+		extended_security = 0;
+	else if (c == '1' || c == 'y' || c == 'Y')
+		extended_security = 1;
+
+	return count;
+}
+
+
 #endif
