@@ -50,6 +50,7 @@ int cifsERROR = 1;
 int traceSMB = 0;
 unsigned int oplockEnabled = 0;
 unsigned int multiuser_mount = 0;
+unsigned int extended_security = 0;
 
 extern int cifs_mount(struct super_block *, struct cifs_sb_info *, char *);
 extern int cifs_umount(struct super_block *, struct cifs_sb_info *);
@@ -340,7 +341,9 @@ static void __exit exit_cifs(void)
 }
 
 MODULE_AUTHOR("Steve French <sfrench@us.ibm.com>");
+#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,7))
 MODULE_LICENSE("GPL");		/* combination of LGPL + GPL source behaves as GPL */
+#endif
 MODULE_DESCRIPTION
     ("VFS support for Common Internet File System servers such as Windows, Samba and those that comply with the SNIA CIFS Specification");
 module_init(init_cifs)
