@@ -30,7 +30,7 @@
 #define MAX_TREE_SIZE 2 + MAX_SERVER_SIZE + 1 + MAX_SHARE_SIZE + 1
 #define MAX_SERVER_SIZE 15
 #define MAX_SHARE_SIZE  64	/* used to be 20 - this should still be enough */
-#define MAX_USERNAME_SIZE 22	/* 22 is to allow for 10 char names + null
+#define MAX_USERNAME_SIZE 32	/* 22 is to allow for 15 char names + null
 				   termination then *2 for unicode versions */
 #define MAX_PASSWORD_SIZE 16
 
@@ -80,6 +80,15 @@ enum statusEnum {
 	CifsNeedReconnect
 };
 
+enum securityEnum {
+	NTLM = 0,  /* Legacy NTLM012 auth with NTLM hash */
+	NTLMv2,    /* Legacy NTLM auth with NTLMv2 hash */
+	RawNTLMSSP,   /* NTLMSSP without SPNEGO */
+    NTLMSSP,  /* NTLMSSP via SPNEGO */
+	Kerberos  /* Kerberos via SPNEGO */
+};
+
+
 
 /*
  *****************************************************************
@@ -106,6 +115,7 @@ struct TCP_Server_Info {
 	enum statusEnum tcpStatus;	/* what we think the status is */
 	struct semaphore tcpSem;
 	struct task_struct *tsk;
+    char server_GUID[16];
 };
 
 /*
@@ -134,6 +144,7 @@ struct cifsSesInfo {
 	enum statusEnum status;
 	int dialectIndex;	/* the negotiated dialect index */
 	char secMode;
+    enum securityEnum secType;
 	unsigned int maxReq;	/* Clients should submit no more */
 	/* than maxReq distinct unanswered SMBs to the server when using  */
 	/* multiplexed reads or writes */
@@ -300,3 +311,4 @@ GLOBAL_EXTERN atomic_t midCount;
 GLOBAL_EXTERN unsigned int multiuser_mount;	/* if enabled allows new sessions to be established on existing mount if we */
 					    /* have the uid/password or Kerberos credential or equivalent for current user */
 GLOBAL_EXTERN unsigned int oplockEnabled;
+GLOBAL_EXTERN unsigned int extended_security;	/* if on, session setup sent with more secure ntlmssp2 challenge/resp */
