@@ -304,7 +304,8 @@ typedef struct smb_hdr {
 	__u8 Flags;
 	__u16 Flags2;		/* note: le */
 	__u16 PidHigh;		/* note: le */
-	__u8 pad[10];
+	__u8 SecuritySignature[8];	/* note le */
+	__u8 pad[2];
 	__u16 Tid;
 	__u16 Pid;		/* note: le */
 	__u16 Uid;
@@ -372,10 +373,16 @@ typedef struct negotiate_rsp {
 	__u16 ServerTimeZone;
 	__u8 EncryptionKeyLength;
 	__u16 ByteCount;
-	unsigned char EncryptionKey[1];	/* if cap extended security is off */
-	/* followed by Domain name - if extended security is off */
-	/* followed by 16 bytes of server GUID */
-	/* followed by security blob if cap_extended_security negotiated */
+	union {
+		unsigned char EncryptionKey[1];	/* if cap extended security is off */
+		/* followed by Domain name - if extended security is off */
+		/* followed by 16 bytes of server GUID */
+		/* followed by security blob if cap_extended_security negotiated */
+		struct {
+			unsigned char GUID[16];
+			unsigned char SecurityBlob[1];
+		} extended_response;
+	} u;
 } NEGOTIATE_RSP;
 
 /* SecurityMode bits */
@@ -534,6 +541,7 @@ typedef struct smb_com_tconx_rsp {
 
 /* tree connect Flags */
 #define DISCONNECT_TID          0x0001
+#define TCON_EXTENDED_SECINFO   0x0008
 /* OptionalSupport bits */
 #define SMB_SUPPORT_SEARCH_BITS 0x0001	/* must have bits (exclusive searches suppt. */
 #define SMB_SHARE_IS_IN_DFS     0x0002
@@ -1474,6 +1482,13 @@ typedef struct {
 	FEA list[1];
 } FEALIST;
 
+/* used to hold an arbitrary blob of data */
+typedef struct data_blob {
+	__u8 *data;
+	size_t length;
+	void (*free) (struct data_blob * data_blob);
+} DATA_BLOB;
+
 #pragma pack()			/* resume default structure packing */
 
 #endif				/* _CIFSPDU_H */
