@@ -47,6 +47,7 @@ extern int SendReceive(const unsigned int /* xid */ , struct cifsSesInfo *,
 extern int checkSMBhdr(struct smb_hdr *smb, __u16 mid);
 extern int checkSMB(struct smb_hdr *smb, __u16 mid, int length);
 extern int smbCalcSize(struct smb_hdr *ptr);
+extern int decode_negTokenInit(unsigned char *security_blob, int length, enum securityEnum * secType);
 extern int map_smb_to_linux_error(struct smb_hdr *smb);
 extern void header_assemble(struct smb_hdr *, char /* command */ ,
 			    const struct cifsTconInfo *, int
@@ -69,12 +70,21 @@ extern int cifs_get_inode_info_unix(struct inode **pinode,
 
 extern int CIFSSMBNegotiate(unsigned int xid, struct cifsSesInfo *ses,
 			    char cryptokey[CIFS_CRYPTO_KEY_SIZE]);
-extern int CIFSSMBSessSetup(unsigned int xid, struct cifsSesInfo *ses,
+extern int CIFSSessSetup(unsigned int xid, struct cifsSesInfo *ses,
 			    char *user, char *domain,
-			    char session_key[CIFS_SESSION_KEY_SIZE],
-			    char session_key2[CIFS_SESSION_KEY_SIZE],
+			    char * session_key, char * ntlm_session_key,
 			    const struct nls_table *);
-extern int CIFSSMBTCon(unsigned int xid, struct cifsSesInfo *ses,
+extern int CIFSSpnegoSessSetup(unsigned int xid, struct cifsSesInfo *ses,
+			      char *user, char *domain,
+			      char *SecurityBlob,
+			      int SecurityBlobLength,
+			      const struct nls_table *);
+extern int CIFSNTLMSSPNegotiateSessSetup(unsigned int xid, struct cifsSesInfo *ses,
+			      char *domain, char * ntlm_session_key, const struct nls_table *);
+extern int CIFSNTLMSSPAuthSessSetup(unsigned int xid, struct cifsSesInfo *ses,
+			      char * user, char *domain, char * ntlm_session_key, char * lanman_session_key, const struct nls_table *);
+
+extern int CIFSTCon(unsigned int xid, struct cifsSesInfo *ses,
 		       const char *tree, struct cifsTconInfo *tcon,
 		       const struct nls_table *);
 
@@ -199,7 +209,7 @@ extern struct cifsTconInfo *tconInfoAlloc(void);
 extern void tconInfoFree(struct cifsTconInfo *);
 
 extern int cifs_demultiplex_thread(struct TCP_Server_Info *);
-
+extern int cifs_reconnect(struct TCP_Server_Info *server);
 
 /* BB routines below not implemented yet BB */
 
