@@ -48,11 +48,11 @@ char *build_path_from_dentry(struct dentry *direntry)
 
 	for (temp = direntry; !IS_ROOT(temp);) {
 		namelen += (1 + temp->d_name.len);
-        cFYI(1,(" len %d ",namelen));
+		cFYI(1, (" len %d ", namelen));
 		temp = temp->d_parent;
 	}
 	namelen += 1;		/* allow for trailing null */
-    cFYI(1,("Final namelength (in build_path): %d ",namelen)); /* BB remove */
+	cFYI(1, ("Final namelength (in build_path): %d ", namelen));	/* BB remove */
 	full_path = kmalloc(namelen, GFP_KERNEL);
 	namelen--;
 	full_path[namelen] = 0;	/* trailing null */
@@ -64,7 +64,7 @@ char *build_path_from_dentry(struct dentry *direntry)
 			full_path[namelen] = '\\';
 			strncpy(full_path + namelen + 1, temp->d_name.name,
 				temp->d_name.len);
-            cFYI(1,(" name: %s ",full_path + namelen)); /* BB remove */
+			cFYI(1, (" name: %s ", full_path + namelen));	/* BB remove */
 		}
 		temp = temp->d_parent;
 	}
@@ -123,7 +123,7 @@ int cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 	int oplock = FALSE;	/* no need to oplock when we are not going to read from the file */
 	__u16 fileHandle;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
 	struct inode *newinode = NULL;
 
@@ -172,7 +172,7 @@ struct dentry *cifs_lookup(struct inode *parent_dir_inode,
 {
 	int rc, xid;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	struct inode *newInode = NULL;
 	char *full_path = NULL;
 
@@ -235,7 +235,7 @@ int cifs_dir_open(struct inode *inode, struct file *file)
 	int rc = 0;
 	int xid;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
 
 
@@ -262,7 +262,10 @@ static int cifs_d_revalidate(struct dentry *direntry, int flags)
 	if (direntry->d_inode) {
 		cFYI(1,
 		     ("In cifs_d_revalidate, name = %s and inode = 0x%p with count %d with time %ld and dentry 0x%p with time %ld\n",
-		      direntry->d_name.name, direntry->d_inode, direntry->d_inode->i_count, direntry->d_inode->i_atime, direntry, direntry->d_time));
+		      direntry->d_name.name, direntry->d_inode,
+		      direntry->d_inode->i_count.counter,
+		      direntry->d_inode->i_atime, direntry,
+		      direntry->d_time));
 	} else {
 		cFYI(1,
 		     ("In cifs_d_revalidate with no inode but name = %s and dentry 0x%p\n",
@@ -280,7 +283,7 @@ static int cifs_d_revalidate(struct dentry *direntry, int flags)
 	return rc;
 }     */
 
-static struct dentry_operations cifs_dentry_ops = {
+struct dentry_operations cifs_dentry_ops = {
 	d_revalidate:cifs_d_revalidate,
 	/* d_delete:       cifs_d_delete,       */
 	/* no need for d_hash, d_compare, d_release, d_iput ... yet. BB confirm this BB */
