@@ -23,6 +23,7 @@
 #include <linux/fs.h>
 #include <linux/stat.h>
 #include <linux/fcntl.h>
+#include <linux/version.h>
 #include <asm/div64.h>
 #include "cifsfs.h"
 #include "cifspdu.h"
@@ -37,7 +38,7 @@ int cifs_open(struct inode *inode, struct file *file)
 	int rc = -EACCES;
 	int xid, oplock;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
 	int desiredAccess = 0x20197;
 	int disposition = FILE_OPEN;
@@ -133,7 +134,7 @@ int cifs_close(struct inode *inode, struct file *file)
 	int rc = 0;
 	int xid;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	struct cifsFileInfo *pSMBFile =
 	    (struct cifsFileInfo *) file->private_data;
 
@@ -168,7 +169,7 @@ int cifs_closedir(struct inode *inode, struct file *file)
 	xid = GetXid();
 
 	if (pSMBFileStruct) {
-        cFYI(1,("\nFreeing private data in close dir"));
+		cFYI(1, ("\nFreeing private data in close dir"));
 		kfree(file->private_data);
 		file->private_data = NULL;
 	}
@@ -185,7 +186,7 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 	__u32 numUnlock = 0;
 	__u64 length;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	length = 1 + pfLock->fl_end - pfLock->fl_start;
 
 	rc = -EACCES;
@@ -239,7 +240,11 @@ int cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 		return -EBADF;
 	}
 
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,9))
+	if(cmd == F_GETLK) {
+#else
 	if (IS_GETLK(cmd)) {
+#endif
 		rc = CIFSSMBLock(xid, pTcon,
 				 ((struct cifsFileInfo *) file->
 				  private_data)->netfid,
@@ -288,16 +293,16 @@ ssize_t cifs_write(struct file * file, const char *write_data,
 	int bytes_written = 0;
 	int total_written;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	int xid, long_op;
 	xid = GetXid();
 
 	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
 	pTcon = cifs_sb->tcon;
 
-	cFYI(1,
+	/*cFYI(1,
 	     (" write %d bytes to offset %lld of %s \n", write_size,
-	      *poffset, file->f_dentry->d_name.name));
+	      *poffset, file->f_dentry->d_name.name)); */
 
 	if (file->private_data == NULL) {
 		FreeXid(xid);
@@ -330,7 +335,11 @@ ssize_t cifs_write(struct file * file, const char *write_data,
 		long_op = FALSE;	/* subsequent writes should be fast - 15 seconds is plenty of time */
 	}
 	file->f_dentry->d_inode->i_ctime =
-	    file->f_dentry->d_inode->i_mtime = CURRENT_TIME;
+	    file->f_dentry->d_inode->i_mtime = CURRENT_TIME;  /* BB is this right? */
+    if(bytes_written > 0) {
+        if (*poffset > file->f_dentry->d_inode->i_size)
+            file->f_dentry->d_inode->i_size = *poffset;
+    }
 	mark_inode_dirty_sync(file->f_dentry->d_inode);
 	return total_written;
 }
@@ -354,7 +363,7 @@ ssize_t cifs_read(struct file * file, char *read_data, size_t read_size,
 	int bytes_read = 0;
 	int total_read;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	int xid;
 
 	xid = GetXid();
@@ -393,17 +402,19 @@ void fill_in_inode(struct inode *tmp_inode,
 {
 	struct cifsInodeInfo *cifsInfo =
 	    (struct cifsInodeInfo *) &(tmp_inode->u);
-
+    pfindData->ExtFileAttributes = le32_to_cpu(pfindData->ExtFileAttributes);
+    pfindData->AllocationSize = le64_to_cpu(pfindData->AllocationSize);
+    pfindData->EndOfFile = le64_to_cpu(pfindData->EndOfFile);
 	cifsInfo->cifsAttrs = pfindData->ExtFileAttributes;
 	cifsInfo->time = jiffies;
-	atomic_inc(&cifsInfo->inUse);  /* inc on every refresh of inode info */
+	atomic_inc(&cifsInfo->inUse);	/* inc on every refresh of inode info */
 
 	/* Linux can not store file creation time unfortunately so we ignore it */
-	tmp_inode->i_atime = cifs_NTtimeToUnix(pfindData->LastAccessTime);
-	tmp_inode->i_mtime = cifs_NTtimeToUnix(pfindData->LastWriteTime);
-	tmp_inode->i_ctime = cifs_NTtimeToUnix(pfindData->ChangeTime);
-    /* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
-	tmp_inode->i_mode = S_IALLUGO & ~(S_ISUID | S_IXGRP); /* 2767 perms - indicate mandatory locking */
+	tmp_inode->i_atime = le64_to_cpu(cifs_NTtimeToUnix(pfindData->LastAccessTime));
+	tmp_inode->i_mtime = le64_to_cpu(cifs_NTtimeToUnix(pfindData->LastWriteTime));
+	tmp_inode->i_ctime = le64_to_cpu(cifs_NTtimeToUnix(pfindData->ChangeTime));
+	/* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
+	tmp_inode->i_mode = S_IALLUGO & ~(S_ISUID | S_IXGRP);	/* 2767 perms - indicate mandatory locking */
 	cFYI(0,
 	     ("\nCIFS FFIRST: Attributes came in as 0x%x",
 	      pfindData->ExtFileAttributes));
@@ -412,25 +423,26 @@ void fill_in_inode(struct inode *tmp_inode,
 		tmp_inode->i_mode |= S_IFLNK;	/* BB can this and S_IFREG or S_IFDIR be set at same time as in Windows? */
 	} else if (pfindData->ExtFileAttributes & ATTR_DIRECTORY) {
 		*pobject_type = DT_DIR;
-        tmp_inode->i_mode = S_IRWXUGO; /* override default perms since we do not lock dirs */
+		tmp_inode->i_mode = S_IRWXUGO;	/* override default perms since we do not lock dirs */
 		tmp_inode->i_mode |= S_IFDIR;
 	} else {
 		*pobject_type = DT_REG;
 		tmp_inode->i_mode |= S_IFREG;
 	}			/* BB add code here - to validate if device or weird share type or odd device type? BB */
 
-    /* can not fill in nlink here as we can in both qpathinfo versions and in unix search */
+	/* can not fill in nlink here as we can in both qpathinfo versions and in unix search */
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
 	    do_div(pfindData->AllocationSize, tmp_inode->i_blksize);
-	cFYI(0,
+	cFYI(1,
 	     ("\nFinddata alloc size (from smb) %lld",
 	      pfindData->AllocationSize));
 	if (pfindData->AllocationSize < pfindData->EndOfFile)
-		cFYI(0, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
+		cFYI(1, ("\nServer inconsistency Error: it says allocation size less than end of file "));	/* BB remove */
 	cFYI(1,
-	     ("\nCIFS FFIRST: Size %ld and blocks %ld ",
-	      (unsigned long) tmp_inode->i_size, tmp_inode->i_blocks));
+	     ("\nCIFS FFIRST: Size %ld and blocks %ld and blocksize %ld",
+	      (unsigned long) tmp_inode->i_size, tmp_inode->i_blocks,
+	      tmp_inode->i_blksize));
 	if (S_ISREG(tmp_inode->i_mode)) {
 		cFYI(1, (" File inode "));
 		tmp_inode->i_op = &cifs_file_inode_ops;
@@ -448,7 +460,7 @@ void fill_in_inode(struct inode *tmp_inode,
 				   kdev_t_to_nr(tmp_inode->i_rdev));
 	}
 }
-
+/* BB make endian neutral */
 void unix_fill_in_inode(struct inode *tmp_inode,
 			FILE_UNIX_INFO * pfindData, int *pobject_type)
 {
@@ -457,17 +469,18 @@ void unix_fill_in_inode(struct inode *tmp_inode,
 	cifsInfo->time = jiffies;
 	atomic_inc(&cifsInfo->inUse);
 
-	tmp_inode->i_atime = cifs_NTtimeToUnix(pfindData->LastAccessTime);
-	tmp_inode->i_mtime =
-	    cifs_NTtimeToUnix(pfindData->LastModificationTime);
-	tmp_inode->i_ctime =
-	    cifs_NTtimeToUnix(pfindData->LastStatusChange);
+	tmp_inode->i_atime = le64_to_cpu(cifs_NTtimeToUnix(pfindData->LastAccessTime));
+	tmp_inode->i_mtime = le64_to_cpu(
+	    cifs_NTtimeToUnix(pfindData->LastModificationTime));
+	tmp_inode->i_ctime = le64_to_cpu(
+	    cifs_NTtimeToUnix(pfindData->LastStatusChange));
 
 	/* tmp_inode->i_mtime =
 	   cifs_NTtimeToUnix(pfindData->LastModificationTime);
 	   tmp_inode->i_ctime =
 	   cifs_NTtimeToUnix(pfindData->LastStatusChange); */
-	tmp_inode->i_mode = pfindData->Permissions;
+	tmp_inode->i_mode = le64_to_cpu(pfindData->Permissions);
+    pfindData->Type = le32_to_cpu(pfindData->Type);
 	if (pfindData->Type == UNIX_FILE) {
 		*pobject_type = DT_REG;
 		tmp_inode->i_mode |= S_IFREG;
@@ -491,10 +504,12 @@ void unix_fill_in_inode(struct inode *tmp_inode,
 		tmp_inode->i_mode |= S_IFSOCK;
 	}
 
-	tmp_inode->i_uid = pfindData->Uid;
-	tmp_inode->i_gid = pfindData->Gid;
-	tmp_inode->i_nlink = pfindData->Nlinks;
+	tmp_inode->i_uid = le64_to_cpu(pfindData->Uid);
+	tmp_inode->i_gid = le64_to_cpu(pfindData->Gid);
+	tmp_inode->i_nlink = le64_to_cpu(pfindData->Nlinks);
 
+    pfindData->NumOfBytes = le64_to_cpu(pfindData->NumOfBytes);
+    pfindData->EndOfFile = le64_to_cpu(pfindData->EndOfFile);
 	tmp_inode->i_size = pfindData->EndOfFile;
 	tmp_inode->i_blocks =
 	    do_div(pfindData->NumOfBytes, tmp_inode->i_blksize);
@@ -521,70 +536,77 @@ void unix_fill_in_inode(struct inode *tmp_inode,
 	}
 }
 
-void construct_dentry(struct qstr * qstring,struct file * file, struct inode ** ptmp_inode,struct dentry ** pnew_dentry)
+void construct_dentry(struct qstr *qstring, struct file *file,
+		      struct inode **ptmp_inode,
+		      struct dentry **pnew_dentry)
 {
-        struct dentry *tmp_dentry;  
-        struct cifs_sb_info *cifs_sb;
-        struct smbTconInfo *pTcon;
+	struct dentry *tmp_dentry;
+	struct cifs_sb_info *cifs_sb;
+	struct cifsTconInfo *pTcon;
 
-        cFYI(1,("\nFor %s ",qstring->name));
-        cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
-        pTcon = cifs_sb->tcon;
+	cFYI(1, ("\nFor %s ", qstring->name));
+	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
+	pTcon = cifs_sb->tcon;
 
-        qstring->hash = full_name_hash(qstring->name, qstring->len);
-        tmp_dentry = d_lookup(file->f_dentry, qstring);
-        if (tmp_dentry) {
-	            cFYI(1, (" existing dentry with inode 0x%p ", tmp_dentry->d_inode));	/* BB remove */
-	            *ptmp_inode = tmp_dentry->d_inode;
-	    /* BB overwrite the old name? i.e. tmp_dentry->d_name and tmp_dentry->d_name.len ?? */
-        } else {
-	            tmp_dentry = d_alloc(file->f_dentry, qstring);
-	            *ptmp_inode = new_inode(file->f_dentry->d_sb);
-                cFYI(1,("\nAlloc new inode %p ",*ptmp_inode));
-	            tmp_dentry->d_op = &cifs_dentry_ops;
-	            cFYI(1, (" instantiate dentry 0x%p with inode 0x%p ",
-	            tmp_dentry, *ptmp_inode));
-	            d_instantiate(tmp_dentry, *ptmp_inode);
-                d_rehash(tmp_dentry);
-        }
+	qstring->hash = full_name_hash(qstring->name, qstring->len);
+	tmp_dentry = d_lookup(file->f_dentry, qstring);
+	if (tmp_dentry) {
+		cFYI(1, (" existing dentry with inode 0x%p ", tmp_dentry->d_inode));	/* BB remove */
+		*ptmp_inode = tmp_dentry->d_inode;
+		/* BB overwrite the old name? i.e. tmp_dentry->d_name and tmp_dentry->d_name.len ?? */
+	} else {
+		tmp_dentry = d_alloc(file->f_dentry, qstring);
+		*ptmp_inode = new_inode(file->f_dentry->d_sb);
+		cFYI(1, ("\nAlloc new inode %p ", *ptmp_inode));
+		tmp_dentry->d_op = &cifs_dentry_ops;
+		cFYI(1, (" instantiate dentry 0x%p with inode 0x%p ",
+			 tmp_dentry, *ptmp_inode));
+		d_instantiate(tmp_dentry, *ptmp_inode);
+		d_rehash(tmp_dentry);
+	}
 
-        tmp_dentry->d_time = jiffies;
-        (*ptmp_inode)->i_blksize =
-                (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00; 
-        *pnew_dentry = tmp_dentry;
+	tmp_dentry->d_time = jiffies;
+	(*ptmp_inode)->i_blksize =
+	    (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
+	cFYI(1, ("\ni_blksize = %ld", (*ptmp_inode)->i_blksize));
+	*pnew_dentry = tmp_dentry;
 }
 
-void cifs_filldir(struct qstr * pqstring, FILE_DIRECTORY_INFO * pfindData, struct file * file, filldir_t filldir, void * direntry)
+void cifs_filldir(struct qstr *pqstring, FILE_DIRECTORY_INFO * pfindData,
+		  struct file *file, filldir_t filldir, void *direntry)
 {
-    struct inode * tmp_inode;
-    struct dentry * tmp_dentry;
-    int object_type;
+	struct inode *tmp_inode;
+	struct dentry *tmp_dentry;
+	int object_type;
 
-    pqstring->name = pfindData->FileName;
-    pqstring->len = pfindData->FileNameLength;
+	pqstring->name = pfindData->FileName;
+	pqstring->len = pfindData->FileNameLength;
 
-    construct_dentry(pqstring, file,&tmp_inode,&tmp_dentry);
+	construct_dentry(pqstring, file, &tmp_inode, &tmp_dentry);
 
-    fill_in_inode(tmp_inode, pfindData, &object_type);
-    filldir(direntry,pfindData->FileName,pqstring->len, file->f_pos, tmp_inode->i_ino, object_type); 
-    dput(tmp_dentry);
+	fill_in_inode(tmp_inode, pfindData, &object_type);
+	filldir(direntry, pfindData->FileName, pqstring->len, file->f_pos,
+		tmp_inode->i_ino, object_type);
+	dput(tmp_dentry);
 }
 
-void cifs_filldir_unix(struct qstr * pqstring, FILE_UNIX_INFO * pUnixFindData, struct file * file, filldir_t filldir, void * direntry)
+void cifs_filldir_unix(struct qstr *pqstring,
+		       FILE_UNIX_INFO * pUnixFindData, struct file *file,
+		       filldir_t filldir, void *direntry)
 {
-    struct inode * tmp_inode;
-    struct dentry * tmp_dentry;
-    int object_type;
+	struct inode *tmp_inode;
+	struct dentry *tmp_dentry;
+	int object_type;
 
-    pqstring->name = pUnixFindData->FileName;
-    pqstring->len =
-        strnlen(pUnixFindData->FileName, MAX_PATHCONF);
+	pqstring->name = pUnixFindData->FileName;
+	pqstring->len = strnlen(pUnixFindData->FileName, MAX_PATHCONF);
 
-    construct_dentry(pqstring, file,&tmp_inode,&tmp_dentry);
+	construct_dentry(pqstring, file, &tmp_inode, &tmp_dentry);
 
 	unix_fill_in_inode(tmp_inode, pUnixFindData, &object_type);
-    filldir(direntry,pUnixFindData->FileName,pqstring->len, file->f_pos, tmp_inode->i_ino, object_type); 
-    dput(tmp_dentry);
+	filldir(direntry, pUnixFindData->FileName, pqstring->len,
+		file->f_pos, tmp_inode->i_ino, object_type);
+	dput(tmp_dentry);
 }
 
 
@@ -598,8 +620,8 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 	int UnixSearch = FALSE;
 	__u16 searchHandle;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
-    struct cifsFileInfo * cifsFile = NULL;
+	struct cifsTconInfo *pTcon;
+	struct cifsFileInfo *cifsFile = NULL;
 	char *full_path = NULL;
 	char *data;
 	struct qstr qstring;
@@ -615,21 +637,21 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 
 	cifs_sb = (struct cifs_sb_info *) (&(file->f_dentry->d_sb->u));
 	pTcon = cifs_sb->tcon;
-    
+
 	full_path = build_wildcard_path_from_dentry(file->f_dentry);
 
 	cFYI(1,
 	     ("\nFull path: %s start at: %lld ", full_path, file->f_pos));
-    
 
-	switch ((int)file->f_pos) {            
+
+	switch ((int) file->f_pos) {
 	case 0:
 		if (filldir
-		    (direntry, ".", 1, file->f_pos, file->f_dentry->d_inode->i_ino,
-		     DT_DIR) < 0) {
+		    (direntry, ".", 1, file->f_pos,
+		     file->f_dentry->d_inode->i_ino, DT_DIR) < 0) {
 			cERROR(1, ("\nFilldir for current dir failed "));
 			break;
-        }
+		}
 		file->f_pos++;
 		/* fallthrough */
 	case 1:
@@ -642,8 +664,8 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 		}
 		file->f_pos++;
 		/* fallthrough */
-    case 2:
-            /* Should we first check if file->private_data is null? */
+	case 2:
+		/* Should we first check if file->private_data is null? */
 		rc = CIFSFindFirst(xid, pTcon, full_path, pfindData,
 				   &findParms, cifs_sb->local_nls,
 				   &Unicode, &UnixSearch);
@@ -659,14 +681,17 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 			if (file->private_data) {
 				memset(file->private_data, 0,
 				       sizeof(struct cifsFileInfo));
-                cifsFile = (struct cifsFileInfo *)file->private_data;
-                cifsFile->netfid = searchHandle;
+				cifsFile =
+				    (struct cifsFileInfo *) file->
+				    private_data;
+				cifsFile->netfid = searchHandle;
 			}
 
 			renew_parental_timestamps(file->f_dentry);
 
-			for (i = 2; i < findParms.SearchCount + 2; i++) {
+			for (i = 2; i < findParms.SearchCount + 2; i++) {                
 				if (UnixSearch == FALSE) {
+                    pfindData->FileNameLength = le32_to_cpu(pfindData->FileNameLength);                   
 					if (Unicode == TRUE)
 						pfindData->FileNameLength =
 						    cifs_strfromUCS_le
@@ -676,131 +701,190 @@ int cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 						     (pfindData->
 						      FileNameLength) / 2,
 						     cifs_sb->local_nls);
-                    qstring.len = pfindData->FileNameLength;
-                    if (((qstring.len != 1)
-                         || (pfindData->FileName[0] != '.'))
-                        && ((qstring.len != 2)
-                        || (pfindData->FileName[0] != '.')
-                        || (pfindData->FileName[1] != '.'))) {
-                        cifs_filldir(&qstring, pfindData, file, filldir, direntry);
-                        file->f_pos++;
-                    }
+					qstring.len =
+					    pfindData->FileNameLength;
+					if (((qstring.len != 1)
+					     || (pfindData->FileName[0] !=
+						 '.'))
+					    && ((qstring.len != 2)
+						|| (pfindData->
+						    FileName[0] != '.')
+						|| (pfindData->
+						    FileName[1] != '.'))) {
+						cifs_filldir(&qstring,
+							     pfindData,
+							     file, filldir,
+							     direntry);
+						file->f_pos++;
+					}
 				} else {	/* UnixSearch */
 					pfindDataUnix =
-					    (FILE_UNIX_INFO *)pfindData;
+					    (FILE_UNIX_INFO *) pfindData;                  
 					if (Unicode == TRUE)
-						qstring.len = cifs_strfromUCS_le
+						qstring.len =
+						    cifs_strfromUCS_le
 						    (pfindDataUnix->
-						     FileName,
-						     (UniChar *)
+						     FileName, (UniChar *)
 						     pfindDataUnix->
 						     FileName,
 						     MAX_PATHCONF,
 						     cifs_sb->local_nls);
-                    else
-                        qstring.len = strnlen(pfindDataUnix->FileName, MAX_PATHCONF);
-                    if (((qstring.len != 1)
-                         || (pfindDataUnix->FileName[0] != '.'))
-                        && ((qstring.len != 2)
-                        || (pfindDataUnix->FileName[0] != '.')
-                        || (pfindDataUnix->FileName[1] != '.'))) {
-                        cifs_filldir_unix(&qstring, pfindDataUnix, file, filldir, direntry);
-                        file->f_pos++;
-                    }
-				}              
-				pfindData = (FILE_DIRECTORY_INFO *) ((char *) pfindData + pfindData->NextEntryOffset);	/* works also for Unix find struct since this is the first field of both */
+					else
+						qstring.len =
+						    strnlen(pfindDataUnix->
+							    FileName,
+							    MAX_PATHCONF);
+					if (((qstring.len != 1)
+					     || (pfindDataUnix->
+						 FileName[0] != '.'))
+					    && ((qstring.len != 2)
+						|| (pfindDataUnix->
+						    FileName[0] != '.')
+						|| (pfindDataUnix->
+						    FileName[1] != '.'))) {
+						cifs_filldir_unix(&qstring,
+								  pfindDataUnix,
+								  file,
+								  filldir,
+								  direntry);
+						file->f_pos++;
+					}
+				}
+				pfindData = (FILE_DIRECTORY_INFO *) ((char *) pfindData + le32_to_cpu(pfindData->NextEntryOffset));	/* works also for Unix find struct since this is the first field of both */
 				/* BB also should check to make sure that pointer is not beyond the end of the SMB */
 			}	/* end for loop */
-            if ((findParms.EndofSearch != 0) && cifsFile) {           
-               cifsFile->endOfSearch = TRUE;
-            }
+			if ((findParms.EndofSearch != 0) && cifsFile) {
+				cifsFile->endOfSearch = TRUE;
+			}
 		} else {
-                if(cifsFile)
-                        cifsFile->endOfSearch = TRUE;
-                rc = 0;	/* unless parent directory disappeared - do not return error here (eg Access Denied or no more files) */            
+			if (cifsFile)
+				cifsFile->endOfSearch = TRUE;
+			rc = 0;	/* unless parent directory disappeared - do not return error here (eg Access Denied or no more files) */
 		}
-        break;
-    default:
-        if (file->private_data == NULL) {      
-                rc = -EBADF;
-                cFYI(1,("\nReaddir on closed srch, pos = %lld",file->f_pos));                
-        } else {	
-                cifsFile = (struct cifsFileInfo *)file->private_data;
-                if(cifsFile->endOfSearch)
-                {
-                        rc = 0;
-                        cFYI(1,("\nEnd of search "));
-                        break;
-                }
-                searchHandle = cifsFile->netfid;                        
-                rc = CIFSFindNext(xid, pTcon, pfindData,&findNextParms, searchHandle, 0,&Unicode, &UnixSearch);
-        		cFYI(1,
-        		     ("Count: %d  End: %d ", findNextParms.SearchCount,
-        		      findNextParms.EndofSearch));       
-                if ((rc == 0) && (findNextParms.SearchCount != 0)) {
-                    for (i = 0; i < findNextParms.SearchCount;i++) {
-                            if (UnixSearch == FALSE) {
-                                if (Unicode == TRUE)
-                                    pfindData->FileNameLength =
-                                        cifs_strfromUCS_le
-                                        (pfindData->FileName,
-                                         (UniChar *)
-                                         pfindData->FileName,
-                                         (pfindData->
-                                          FileNameLength) / 2,
-                                         cifs_sb->local_nls);
-                                qstring.len = pfindData->FileNameLength;
-                                if (((qstring.len != 1)
-                                     || (pfindData->FileName[0] != '.'))
-                                    && ((qstring.len != 2)
-                                    || (pfindData->FileName[0] != '.')
-                                    || (pfindData->FileName[1] != '.'))) {
-                                    cifs_filldir(&qstring, pfindData, file, filldir, direntry);
-                                    file->f_pos++;
-                                }
-                            } else {	/* UnixSearch */
-                                pfindDataUnix =
-                                    (FILE_UNIX_INFO *)pfindData;
-                                if (Unicode == TRUE)
-                                    qstring.len = cifs_strfromUCS_le
-                                        (pfindDataUnix->
-                                         FileName,
-                                         (UniChar *)
-                                         pfindDataUnix->
-                                         FileName,
-                                         MAX_PATHCONF,
-                                         cifs_sb->local_nls);
-                                else
-                                    qstring.len = strnlen(pfindDataUnix->FileName, MAX_PATHCONF);
+		break;
+	default:
+		if (file->private_data == NULL) {
+			rc = -EBADF;
+			cFYI(1,
+			     ("\nReaddir on closed srch, pos = %lld",
+			      file->f_pos));
+		} else {
+			cifsFile =
+			    (struct cifsFileInfo *) file->private_data;
+			if (cifsFile->endOfSearch) {
+				rc = 0;
+				cFYI(1, ("\nEnd of search "));
+				break;
+			}
+			searchHandle = cifsFile->netfid;
+			rc = CIFSFindNext(xid, pTcon, pfindData,
+					  &findNextParms, searchHandle, 0,
+					  &Unicode, &UnixSearch);
+			cFYI(1,
+			     ("Count: %d  End: %d ",
+			      findNextParms.SearchCount,
+			      findNextParms.EndofSearch));
+			if ((rc == 0) && (findNextParms.SearchCount != 0)) {
+				for (i = 0; i < findNextParms.SearchCount;
+				     i++) {
+                    pfindData->FileNameLength = le32_to_cpu(pfindData->FileNameLength); 
+                    if (UnixSearch == FALSE) {
+						if (Unicode == TRUE)
+							pfindData->
+							    FileNameLength
+							    =
+							    cifs_strfromUCS_le
+							    (pfindData->
+							     FileName,
+							     (UniChar *)
+							     pfindData->
+							     FileName,
+							     (pfindData->
+							      FileNameLength)
+							     / 2,
+							     cifs_sb->
+							     local_nls);
+						qstring.len =
+						    pfindData->
+						    FileNameLength;
+						if (((qstring.len != 1)
+						     || (pfindData->
+							 FileName[0] !=
+							 '.'))
+						    && ((qstring.len != 2)
+							|| (pfindData->
+							    FileName[0] !=
+							    '.')
+							|| (pfindData->
+							    FileName[1] !=
+							    '.'))) {
+							cifs_filldir
+							    (&qstring,
+							     pfindData,
+							     file, filldir,
+							     direntry);
+							file->f_pos++;
+						}
+					} else {	/* UnixSearch */
+						pfindDataUnix =
+						    (FILE_UNIX_INFO *)
+						    pfindData;
+						if (Unicode == TRUE)
+							qstring.len =
+							    cifs_strfromUCS_le
+							    (pfindDataUnix->
+							     FileName,
+							     (UniChar *)
+							     pfindDataUnix->
+							     FileName,
+							     MAX_PATHCONF,
+							     cifs_sb->
+							     local_nls);
+						else
+							qstring.len =
+							    strnlen
+							    (pfindDataUnix->
+							     FileName,
+							     MAX_PATHCONF);
 
-                                if (((qstring.len != 1)
-                                     || (pfindDataUnix->FileName[0] != '.'))
-                                    && ((qstring.len != 2)
-                                    || (pfindDataUnix->FileName[0] != '.')
-                                    || (pfindDataUnix->FileName[1] != '.'))) {
-                                    cifs_filldir_unix(&qstring, pfindDataUnix, file, filldir, direntry);
-                                    file->f_pos++;
-                                }
-                            }              
-                            pfindData = (FILE_DIRECTORY_INFO *) ((char *) pfindData + pfindData->NextEntryOffset);	/* works also for Unix find struct since this is the first field of both */
-                            /* BB also should check to make sure that pointer is not beyond the end of the SMB */
+						if (((qstring.len != 1)
+						     || (pfindDataUnix->
+							 FileName[0] !=
+							 '.'))
+						    && ((qstring.len != 2)
+							|| (pfindDataUnix->
+							    FileName[0] !=
+							    '.')
+							|| (pfindDataUnix->
+							    FileName[1] !=
+							    '.'))) {
+							cifs_filldir_unix
+							    (&qstring,
+							     pfindDataUnix,
+							     file, filldir,
+							     direntry);
+							file->f_pos++;
+						}
+					}
+					pfindData = (FILE_DIRECTORY_INFO *) ((char *) pfindData + le32_to_cpu(pfindData->NextEntryOffset));	/* works also for Unix find struct since this is the first field of both */
+					/* BB also should check to make sure that pointer is not beyond the end of the SMB */
 
-                    } /* end for loop */
-                    if (findNextParms.EndofSearch != 0) {           
-                            cifsFile->endOfSearch = TRUE;                    
-                    }
-        		} else {
-                    cifsFile->endOfSearch = TRUE;
-        			rc = 0;	/* unless parent directory disappeared - do not return error here (eg Access Denied or no more files) */            
-        		}
-        }
+				}	/* end for loop */
+				if (findNextParms.EndofSearch != 0) {
+					cifsFile->endOfSearch = TRUE;
+				}
+			} else {
+				cifsFile->endOfSearch = TRUE;
+				rc = 0;	/* unless parent directory disappeared - do not return error here (eg Access Denied or no more files) */
+			}
+		}
 	}			/* end switch */
 
-    if (data)
-	    kfree(data);
-    if (full_path)
-	    kfree(full_path);
-    FreeXid(xid);
+	if (data)
+		kfree(data);
+	if (full_path)
+		kfree(full_path);
+	FreeXid(xid);
 
-    return rc;
-}
\ No newline at end of file
+	return rc;
+}
