@@ -33,13 +33,12 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 			     struct super_block *sb)
 {
 	int xid;
-	int num_referrals;
 	int rc = 0;
 	FILE_UNIX_BASIC_INFO findData;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	struct inode *inode;
 	struct cifs_sb_info *cifs_sb = (struct cifs_sb_info *) &sb->u;
-	unsigned char *referrals;
+	char *tmp_path;
 
 	xid = GetXid();
 
@@ -51,41 +50,60 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 	/* dump_mem("\nUnixQPathInfo return data", &findData, sizeof(findData)); */
 	if (rc) {
 		if (rc == -EREMOTE) {
-			/* rc = */ CIFSGetDFSRefer(xid, pTcon, search_path,
-						   &referrals,
-						   &num_referrals,
-						   cifs_sb->local_nls);
-			/* BB finish up BB */
+/* rc = *//* CIFSGetDFSRefer(xid, pTcon->ses, search_path,
+   &referrals,
+   &num_referrals,
+   cifs_sb->local_nls); */
+			tmp_path =
+			    kmalloc(strnlen
+				    (pTcon->treeName,
+				     MAX_TREE_SIZE + 1) +
+				    strnlen(search_path, MAX_PATHCONF) + 1,
+				    GFP_KERNEL);
+			if (tmp_path == NULL) {
+				FreeXid(xid);
+				return -ENOMEM;
+			}
+
+			strncpy(tmp_path, pTcon->treeName, MAX_TREE_SIZE);	/* have to skip first of the double backslash of UNC name */
+			strncat(tmp_path, search_path, MAX_PATHCONF);
+			rc = connect_to_dfs_path(xid, pTcon->ses,
+						 /* treename + */ tmp_path,
+						 cifs_sb->local_nls);
+			kfree(tmp_path);
+
+			/* BB fix up inode etc. */
 		} else if (rc) {
 			FreeXid(xid);
 			return rc;
 		}
 
 	} else {
-        struct cifsInodeInfo *cifsInfo;
+		struct cifsInodeInfo *cifsInfo;
 
 		/* get new inode */
-		if (*pinode == NULL) {        
+		if (*pinode == NULL) {
 			*pinode = new_inode(sb);
-            cFYI(1,("\nAlloc new inode %p ",*pinode));
-        }
+			cFYI(1, ("\nAlloc new inode %p ", *pinode));
+		}
 		inode = *pinode;
 /*        new_inode = iget(parent_dir_inode->i_sb, findData.IndexNumber); *//* index number not reliable in response data */
 
-        cifsInfo = (struct cifsInodeInfo *) &(inode->u);
+		cifsInfo = (struct cifsInodeInfo *) &(inode->u);
 
-        cFYI(1,("\nOld time %ld ",cifsInfo->time));
-        cifsInfo->time = jiffies;
-        cFYI(1,(" New time %ld ",cifsInfo->time));
-        atomic_inc(&cifsInfo->inUse);  /* inc on every refresh of inode */
+		cFYI(1, ("\nOld time %ld ", cifsInfo->time));
+		cifsInfo->time = jiffies;
+		cFYI(1, (" New time %ld ", cifsInfo->time));
+		atomic_inc(&cifsInfo->inUse);	/* inc on every refresh of inode */
 
-		inode->i_atime =
-		    cifs_NTtimeToUnix(findData.LastAccessTime);
-		inode->i_mtime =
-		    cifs_NTtimeToUnix(findData.LastModificationTime);
-		inode->i_ctime =
-		    cifs_NTtimeToUnix(findData.LastStatusChange);
-		inode->i_mode = findData.Permissions;
+		inode->i_atime = le64_to_cpu(
+		    cifs_NTtimeToUnix(findData.LastAccessTime));
+		inode->i_mtime = le64_to_cpu(
+		    cifs_NTtimeToUnix(findData.LastModificationTime));
+		inode->i_ctime = le64_to_cpu(
+		    cifs_NTtimeToUnix(findData.LastStatusChange));
+		inode->i_mode = le64_to_cpu(findData.Permissions);
+        findData.Type = le32_to_cpu(findData.Type);
 		if (findData.Type == UNIX_FILE) {
 			inode->i_mode |= S_IFREG;
 		} else if (findData.Type == UNIX_SYMLINK) {
@@ -101,9 +119,11 @@ int cifs_get_inode_info_unix(struct inode **pinode,
 		} else if (findData.Type == UNIX_SOCKET) {
 			inode->i_mode |= S_IFSOCK;
 		}
-		inode->i_uid = findData.Uid;
-		inode->i_gid = findData.Gid;
-		inode->i_nlink = findData.Nlinks;
+		inode->i_uid = le64_to_cpu(findData.Uid);
+		inode->i_gid = le64_to_cpu(findData.Gid);
+		inode->i_nlink = le64_to_cpu(findData.Nlinks);
+        findData.NumOfBytes = le64_to_cpu(findData.NumOfBytes);
+        findData.EndOfFile = le64_to_cpu(findData.EndOfFile);
 		inode->i_size = findData.EndOfFile;
 		inode->i_blksize =
 		    (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
@@ -145,12 +165,11 @@ int cifs_get_inode_info(struct inode **pinode,
 {
 	int xid;
 	int rc = 0;
-	int num_referrals;
-	unsigned char *referrals;
 	FILE_ALL_INFO findData;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	struct inode *inode;
 	struct cifs_sb_info *cifs_sb = (struct cifs_sb_info *) &sb->u;
+	char *tmp_path;
 
 	xid = GetXid();
 
@@ -163,42 +182,58 @@ int cifs_get_inode_info(struct inode **pinode,
 	if (rc) {
 		if (rc == -EREMOTE) {
 			/* BB add call to new func rc = GetDFSReferral(); */
-			/* rc = */ CIFSGetDFSRefer(xid, pTcon, search_path,
-						   &referrals,
-						   &num_referrals,
-						   cifs_sb->local_nls);
-			/* BB finish up BB */
+/* rc = *//* CIFSGetDFSRefer(xid, pTcon->ses, search_path,
+   &referrals,
+   &num_referrals,
+   cifs_sb->local_nls); */
+			tmp_path =
+			    kmalloc(strnlen
+				    (pTcon->treeName,
+				     MAX_TREE_SIZE + 1) +
+				    strnlen(search_path, MAX_PATHCONF) + 1,
+				    GFP_KERNEL);
+			if (tmp_path == NULL) {
+				FreeXid(xid);
+				return -ENOMEM;
+			}
+
+			strncpy(tmp_path, pTcon->treeName, MAX_TREE_SIZE);
+			strncat(tmp_path, search_path, MAX_PATHCONF);
+			rc = connect_to_dfs_path(xid, pTcon->ses,
+						 /* treename + */ tmp_path,
+						 cifs_sb->local_nls);
+			kfree(tmp_path);
+			/* BB fix up inode etc. */
 		} else if (rc) {
 			FreeXid(xid);
 			return rc;
 		}
 	} else {
-        struct cifsInodeInfo *cifsInfo;
-		
-        /* get new inode */
-		if (*pinode == NULL) {        
+		struct cifsInodeInfo *cifsInfo;
+
+		/* get new inode */
+		if (*pinode == NULL) {
 			*pinode = new_inode(sb);
-            cFYI(1,("\nAlloc new inode %p ",*pinode));
-        }
+			cFYI(1, ("\nAlloc new inode %p ", *pinode));
+		}
 
 		inode = *pinode;
-        cifsInfo = (struct cifsInodeInfo *) &(inode->u);
-
+		cifsInfo = (struct cifsInodeInfo *) &(inode->u);
+        findData.Attributes = le32_to_cpu(findData.Attributes);
         cifsInfo->cifsAttrs = findData.Attributes;
-        cFYI(1,("\nOld time %ld ",cifsInfo->time));
-        cifsInfo->time = jiffies;
-        cFYI(1,(" New time %ld ",cifsInfo->time));
-        atomic_inc(&cifsInfo->inUse);  /* inc on every refresh of inode */
+		cFYI(1, ("\nOld time %ld ", cifsInfo->time));
+		cifsInfo->time = jiffies;
+		cFYI(1, (" New time %ld ", cifsInfo->time));
+		atomic_inc(&cifsInfo->inUse);	/* inc on every refresh of inode */
 
 		inode->i_blksize =
 		    (pTcon->ses->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
 		/* Linux can not store file creation time unfortunately so we ignore it */
-		inode->i_atime =
-		    cifs_NTtimeToUnix(findData.LastAccessTime);
-		inode->i_mtime = cifs_NTtimeToUnix(findData.LastWriteTime);
-		inode->i_ctime = cifs_NTtimeToUnix(findData.ChangeTime);
+		inode->i_atime = cifs_NTtimeToUnix(le64_to_cpu(findData.LastAccessTime));
+		inode->i_mtime = cifs_NTtimeToUnix(le64_to_cpu(findData.LastWriteTime));
+		inode->i_ctime = cifs_NTtimeToUnix(le64_to_cpu(findData.ChangeTime));
 /* inode->i_mode = S_IRWXUGO;  *//* 777 perms */
-        /* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
+		/* should we treat the dos attribute of read-only as read-only mode bit e.g. 555 */
 		inode->i_mode = S_IALLUGO & ~(S_ISUID | S_IXGRP);	/* 2767 perms indicate mandatory locking - will override for dirs later */
 		cFYI(0,
 		     ("\nAttributes came in as 0x%x\n",
@@ -212,13 +247,13 @@ int cifs_get_inode_info(struct inode **pinode,
 			inode->i_mode |= S_IFREG;
 			/* BB add code here - validate if device or weird share type or device type? BB */
 		}
-		inode->i_size = findData.EndOfFile;
-		inode->i_blocks =
-		    do_div(findData.AllocationSize, inode->i_blksize);
+		inode->i_size = le64_to_cpu(findData.EndOfFile);
+        findData.AllocationSize = le64_to_cpu(findData.AllocationSize);
+		inode->i_blocks = do_div(findData.AllocationSize, inode->i_blksize);
 		cFYI(1,
 		     ("\n Size %ld and blocks %ld ",
 		      (unsigned long) inode->i_size, inode->i_blocks));
-		inode->i_nlink = findData.NumberOfLinks;
+		inode->i_nlink = le32_to_cpu(findData.NumberOfLinks);
 
 		/* BB fill in uid and gid here? with help from winbind? BB */
 
@@ -261,8 +296,9 @@ int cifs_unlink(struct inode *inode, struct dentry *direntry)
 	int rc = 0;
 	int xid;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
+    struct cifsInodeInfo * cifsInode;
 
 
 	cFYI(1, ("\n cifs_unlink, inode = 0x%p with ", inode));
@@ -272,9 +308,21 @@ int cifs_unlink(struct inode *inode, struct dentry *direntry)
 	cifs_sb = (struct cifs_sb_info *) (&(inode->i_sb->u));
 	pTcon = cifs_sb->tcon;
 
+    /* BB Should we close the file if it is already open from our client? */
+
 	full_path = build_path_from_dentry(direntry);
 
 	rc = CIFSSMBDelFile(xid, pTcon, full_path, cifs_sb->local_nls);
+
+    if(!rc) {          
+        direntry->d_inode->i_nlink--;
+    }
+    cifsInode = (struct cifsInodeInfo *)&direntry->d_inode->u;
+    cifsInode->time = 0; /* will force revalidate to go get info when needed */
+    direntry->d_inode->i_ctime = inode->i_ctime = inode->i_mtime = CURRENT_TIME;
+    cifsInode = (struct cifsInodeInfo *)&inode->u;
+    cifsInode->time = 0; /* force revalidate of dir as well */
+
 	if (full_path)
 		kfree(full_path);
 	FreeXid(xid);
@@ -286,11 +334,11 @@ int cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 	int rc = 0;
 	int xid;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
 	struct inode *newinode = NULL;
-
-	cFYI(1, ("In cifs_mkdir, inode = 0x%p\n", inode));
+    	
+    cFYI(1, ("In cifs_mkdir, inode = 0x%p\n", inode));
 
 	xid = GetXid();
 
@@ -303,6 +351,7 @@ int cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 	if (rc) {
 		cFYI(1, ("\ncifs_mkdir returned 0x%x ", rc));
 	} else {
+        inode->i_nlink++;
 		if (pTcon->ses->capabilities & CAP_UNIX)
 			rc = cifs_get_inode_info_unix(&newinode, full_path,
 						      inode->i_sb);
@@ -311,7 +360,8 @@ int cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 						 inode->i_sb);
 
 		d_instantiate(direntry, newinode);
-	}
+        direntry->d_inode->i_nlink = 2;	
+    }
 	if (full_path)
 		kfree(full_path);
 	FreeXid(xid);
@@ -324,9 +374,9 @@ int cifs_rmdir(struct inode *inode, struct dentry *direntry)
 	int rc = 0;
 	int xid;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
-
+    struct cifsInodeInfo * cifsInode;
 
 	cFYI(1, ("\nn cifs_rmdir, inode = 0x%p with ", inode));
 
@@ -338,6 +388,17 @@ int cifs_rmdir(struct inode *inode, struct dentry *direntry)
 	full_path = build_path_from_dentry(direntry);
 
 	rc = CIFSSMBRmDir(xid, pTcon, full_path, cifs_sb->local_nls);
+
+    if(!rc){
+        inode->i_nlink--;
+        direntry->d_inode->i_size = 0;
+        direntry->d_inode->i_nlink = 0;
+    }
+
+    cifsInode = (struct cifsInodeInfo *)&direntry->d_inode->u;
+    cifsInode->time = 0; /* will force revalidate to go get info when needed */
+    direntry->d_inode->i_ctime = inode->i_ctime = inode->i_mtime = CURRENT_TIME;
+
 	if (full_path)
 		kfree(full_path);
 	FreeXid(xid);
@@ -351,7 +412,7 @@ int cifs_rename(struct inode *source_inode, struct dentry *source_direntry,
 	char *toName;
 	struct cifs_sb_info *cifs_sb_source;
 	struct cifs_sb_info *cifs_sb_target;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	int xid;
 	int rc = 0;
 
@@ -385,7 +446,7 @@ int cifs_revalidate(struct dentry *direntry)
 	int rc = 0;
 	char *full_path;
 	struct cifs_sb_info *cifs_sb;
-    struct cifsInodeInfo * cifsInode;
+	struct cifsInodeInfo *cifsInode;
 
 	xid = GetXid();
 
@@ -393,20 +454,21 @@ int cifs_revalidate(struct dentry *direntry)
 
 	full_path = build_path_from_dentry(direntry);
 	cFYI(1,
-	     (" full path: %s for inode 0x%p with count %d dentry: 0x%p d_time %ld at time %ld \n", full_path,direntry->d_inode,direntry->d_inode->i_count, direntry,
+	     (" full path: %s for inode 0x%p with count %d dentry: 0x%p d_time %ld at time %ld \n",
+	      full_path, direntry->d_inode,
+	      direntry->d_inode->i_count.counter, direntry,
 	      direntry->d_time, jiffies));
 
-    cifsInode = (struct cifsInodeInfo *) &(direntry->d_inode->u);
+	cifsInode = (struct cifsInodeInfo *) &(direntry->d_inode->u);
 
 
-    if(time_before(jiffies,cifsInode->time + HZ))
-    {
-            cFYI(1,(" Do not need to revalidate "));
-            if (full_path)
-                kfree(full_path);
-            FreeXid(xid);
-            return rc;
-    }
+	if ((time_before(jiffies, cifsInode->time + HZ)) && (direntry->d_inode->i_nlink == 1)) {
+		cFYI(1, (" Do not need to revalidate "));
+		if (full_path)
+			kfree(full_path);
+		FreeXid(xid);
+		return rc;
+	}
 
 	if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)
 		cifs_get_inode_info_unix(&direntry->d_inode, full_path,
@@ -429,7 +491,7 @@ void cifs_truncate_file(struct inode *inode)
 	int xid;
 	int rc = 0;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	struct dentry *dirent;
 	char *full_path = NULL;
 
@@ -467,7 +529,7 @@ int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 {
 	int xid;
 	struct cifs_sb_info *cifs_sb;
-	struct smbTconInfo *pTcon;
+	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
 	int rc = -EACCES;
 	FILE_BASIC_INFO time_buf;
@@ -475,7 +537,7 @@ int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 	__u64 mode = 0xFFFFFFFFFFFFFFFF;
 	__u64 uid = 0xFFFFFFFFFFFFFFFF;
 	__u64 gid = 0xFFFFFFFFFFFFFFFF;
-
+	struct cifsInodeInfo *cifsInode;
 
 	xid = GetXid();
 
@@ -551,6 +613,8 @@ int cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 		rc = CIFSSMBSetTimes(xid, pTcon, full_path, &time_buf,
 				     cifs_sb->local_nls);
 	}
+    cifsInode = (struct cifsInodeInfo *) &(direntry->d_inode->u);
+    cifsInode->time = 0; /* will force revalidate to go get attributes when needed */
 
 	if (full_path)
 		kfree(full_path);
