@@ -33,7 +33,7 @@
 
 
 struct mid_q_entry *AllocMidQEntry(struct smb_hdr *smb_buffer,
-				   struct smbSesInfo *ses)
+				   struct cifsSesInfo *ses)
 {
 	struct mid_q_entry *temp;
 
@@ -59,7 +59,7 @@ struct mid_q_entry *AllocMidQEntry(struct smb_hdr *smb_buffer,
 		temp->ses = ses;
 		temp->tsk = current;
 	}
-	if (ses->failureFlag == FailNone) {
+	if (ses->status == CifsGood) {
 		list_add_tail(&temp->qhead, &ses->server->pending_mid_q);
 		atomic_inc(&midCount);
 	} else {		/* BB add reconnect code here BB */
@@ -129,7 +129,7 @@ int smb_send(struct socket *ssocket, struct smb_hdr *smb_buffer,
 	return rc;
 }
 
-int SendReceive(const unsigned int xid, struct smbSesInfo *ses,
+int SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 		struct smb_hdr *in_buf, struct smb_hdr *out_buf,
 		int *pbytes_returned, const int long_op)
 {
@@ -195,7 +195,7 @@ int SendReceive(const unsigned int xid, struct smbSesInfo *ses,
 			       4 /* include 4 byte RFC1001 header */ );
 			/* convert the length back to a form that we can use */
 
-			dump_smb(out_buf, 92);	/* BB remove */
+			dump_smb(out_buf, 92);
 
 			out_buf->smb_buf_length =
 			    be32_to_cpu(out_buf->smb_buf_length);
@@ -210,7 +210,13 @@ int SendReceive(const unsigned int xid, struct smbSesInfo *ses,
 
 
 			*pbytes_returned = out_buf->smb_buf_length;
+
+			/* BB special case reconnect tid and reconnect uid here? */
 			rc = map_smb_to_linux_error(out_buf);	/* BB watch endianness here BB */
+
+            /* convert ByteCount if necessary */
+            if(receive_len >= sizeof(struct smb_hdr) - 4 /* do not count RFC1001 header */ + (2 * out_buf->WordCount) + 2 /* bcc */)
+                BCC(out_buf) = le16_to_cpu(BCC(out_buf));
 		} else
 			rc = -EIO;
 	}
